# -*- coding: utf-8 -*-
from qgis.PyQt.QtWidgets import QAction, QMessageBox, QFileDialog, QToolTip
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWebKitWidgets import QWebView, QWebPage
from qgis.PyQt.QtCore import QUrl, QPoint # , Qt, QObject
from qgis.core import QgsVectorLayer, QgsMapLayer, QgsProject
from qgis.utils import iface
#from qgis.gui import *
import os
from .mafenetre import fenetre


class plugin():
	def __init__(self, iface):
		# self.siteWeb est une variable qui sera accessible depuis toute la classe "plugin" :
		self.siteWeb = "http://www.geoinformations.developpement-durable.gouv.fr"
		self.fen = None # L'objet fenêtre pour "ouvrirFenetre"


	def initGui(self): # Pour ajouter les commandes du plugin (boutons, menus...) à l'interface de QGIS
		win = iface.mainWindow()
		pluginPath = os.path.abspath(os.path.dirname(__file__))
		
		self.toolBar = iface.pluginToolBar() # la barre d'outil des Extensions
		self.boutons = [] # Liste des QAction ajoutées par le plugin
		
		bouton1 = QAction( QIcon(pluginPath +'/icons/shp.png'), "Ouvrir un shapefile", win )
		bouton1.triggered.connect( self.ouvrirCouche )
		self.toolBar.addAction( bouton1 )
		self.boutons.append(bouton1)
		
		bouton2= QAction( "Web", win )
		bouton2.setToolTip( "Afficher le site internet de QGIS" )
		bouton2.triggered.connect( self.ouvrirWeb )
		self.toolBar.addAction( bouton2 )
		self.boutons.append(bouton2)
		
		self.bouton3= QAction( "info", win )
		self.bouton3.setToolTip( "informations sur la couche active" )
		self.bouton3.triggered.connect( self.infoCouche )
		self.toolBar.addAction( self.bouton3 )
		
		self.bouton4 = QAction( "Outils", win )
		self.bouton4.setToolTip("ouvrir mes outils")
		self.bouton4.triggered.connect( self.ouvrirFenetre )
		self.toolBar.addAction( self.bouton4 )


	def ouvrirFenetre(self): # Fonction qui ouvre un dialogue défini dans mafenetre.py
		if not self.fen:  self.fen = fenetre() # Créer l'objet
		self.fen.show() # L'afficher


	def ouvrirCouche(self): # Fonction qui ouvre un fichier Shapefile à choisir par l'utilisateur
		fichierShp, exten= QFileDialog.getOpenFileName( iface.mainWindow(), "Choisir le fichier Shapefile à ouvrir",
			"", "Shapefile (*.shp)" )
		if not fichierShp or fichierShp=='':
			return
		nomFichier= os.path.basename(os.path.splitext(fichierShp)[0]) # Fichier sans le chemin ni l'extension
		macouche= QgsVectorLayer(fichierShp, nomFichier, 'ogr') # Créer une couche à partir du fichier
		if not macouche or not macouche.isValid(): # Si QGIS n'a pas pu créer une couche à partir de de fichier
			QMessageBox.warning(iface.mainWindow(), "Echec",
			 "Le fichier \"%s\" n'est pas reconnu par QGIS."% fichierShp.replace('/',os.sep) )
			return
		QgsProject.instance().addMapLayer(macouche) # Ouvrir la couche dans le projet actuel


	def ouvrirWeb(self): # Fonction pour ouvrir le site web : qgis.org
		self.web= QWebView() # Créer la fenêtre pour afficher la page web
		page= QWebPage() # La "zone" dans la fenêtre qui va gérer l'affichage web
		self.web.setPage(page)
		self.web.setUrl( QUrl(self.siteWeb) )
		self.web.show() # Afficher la fenêtre
		# Afficher un petit text d'infos éphémère :
		QToolTip.showText( self.toolBar.mapToGlobal( QPoint( 0, 0 ) ), self.siteWeb )


	def infoCouche(self): # Pour donner qq infos sur la couche active : nom, type, source, champs, etc.
		win= iface.mainWindow()
		layer= iface.activeLayer()
		if not layer:
			QMessageBox.warning(win, "Remarque", "Cet outil donne des informations sur la couche active.\nMais il n'y a pas de couche active dans le projet actuel." )
			return
		
		types= ['Vecteur','Raster (image)','Plugin','Filet (Mesh)','inconnu','inconnu','inconnu']
		type= layer.type()
		info= "Couche : %s\n\nType : %s \n\nSource :\n%s"% ( layer.name(), types[type], layer.source() )
		if type != QgsMapLayer.VectorLayer:
			QMessageBox.about( win, "Informations sur la couche active", info )
			return
		
		info += "\n\nProvider : %s\n\n"% ( layer.providerType() )
		info += "Nb d'entités : %i\n"% ( layer.featureCount() )
		info += "Nb d'entités sélectionnées : %i\n\n"% ( layer.selectedFeatureCount() )
		chps= str( layer.fields().names() )
		listChp= "'"
		info += "Champs :\n%s\n\n"% ( chps )
		info += "Voulez-vous voir les entités de la couche (zoom et attributs) ?"
		# Message d'informations avec une question (2 réponses : Oui ou Non) :
		ret= QMessageBox.information( win, "Informations sur la couche active", info,
		 QMessageBox.No | QMessageBox.Yes, QMessageBox.No )
		if ret != QMessageBox.Yes: return
		
		num= 1
		features= layer.getFeatures() # C'est en quelque sorte la liste des entités de la couche
		for ft in features: # Parcourir les entités de la couche
			layer.removeSelection()
			layer.select( ft.id() ) # Sélectionner l'entité
			iface.mapCanvas().zoomToSelected( layer ) # Zoomer sur l'entité Sélectionnée
			ret= QMessageBox.information( win, "Entité n°%i"% num, str(ft.attributes() ),
			 QMessageBox.Cancel | QMessageBox.Ok, QMessageBox.Ok )
			if ret != QMessageBox.Ok: break
			num+=1


	def unload(self): # Supprimer menus et boutons
		for B in self.boutons:  self.toolBar.removeAction( B )
		self.toolBar.removeAction( self.bouton3 )
		self.toolBar.removeAction( self.bouton4 )

